package jp.gr.java_conf.skrb.game.pong;

public class GameState implements State {
    private final static int MAX_BALL = 3;
    private final static int SCORE_INC = 10;

    private PongGameEngine engine;
    private PadController padController;
    private BallController ballController;
    private PongView view;

    private int numBall;
    private int score;
    private int highScore;
    private boolean gaming;
    private int inputState;
    private int x;
    private int y;

    public GameState(PongGameEngine engine, 
                  PongView view,
                  PadController padController,
                  BallController ballController) {
        this.engine = engine;
        this.view = view;
        this.padController = padController;
        this.ballController = ballController;
    }

    public State execute() {
        InputListener listener = new InputListener() {
                public void gameStart(InputEvent event) {}

                public void gameQuit(InputEvent event) {
                    end();
                }

                public void padMoving(InputEvent event) {
//                    if (event.getID() == InputEvent.TAPPED) {
//                        updatePad(event.getX(), event.getY());
//                    } else {
//                        updatePad(event.getID());
//                    }
                    inputState = event.getID();
                    x = event.getX();
                    y = event.getY();
                }
            };

        view.addInputListener(listener);

        score = 0;
        highScore = engine.getHighScore();
        
        start();
        padController.init();
        for (numBall = 1; numBall <=  MAX_BALL; numBall++) {
            init();
            play();
            if (!isGaming()) {
                break;
            }
        }

        engine.setHighScore(highScore);
        view.removeInputListener(listener);

        if (isGaming()) {
            return new DemoState(engine, view, padController, ballController);
        } else {
            return null;
        }
    }

    private synchronized void end() {
        gaming = false;
    }

    private synchronized void start() {
        gaming = true;
    }

    private synchronized boolean isGaming() {
        return gaming;
    }

    private void init() {
        ballController.init();
        
        view.draw(MAX_BALL - numBall, score, highScore,
                  ballController.getBall(), padController.getPad(), true);
    }

    private void play() {
        try {
            while (isGaming()) {
                Thread.sleep(40L);

                inputState = padController.update(inputState, x, y);

                boolean isSurviving = ballController.update();
                if (ballController.bounceCheck(padController.getBounds())) {
                    score += SCORE_INC;
                    if (highScore <= score) {
                        highScore = score;
                    }
                }

                view.draw(MAX_BALL - numBall, score, highScore,
                          ballController.getBall(), padController.getPad(), true);

                if (!isSurviving) {
                    break;
                }
            }
        } catch (InterruptedException ex) {
            ex.printStackTrace();
        }
    }

/*
    private void updatePad(int id) {
        switch(id) {
        case InputEvent.RIGHT:
            padController.moveRight();
            break;
        case InputEvent.LEFT:
            padController.moveLeft();
            break;
        }
    }

    private void updatePad(int x, int y) {
        int padX = padController.getX();
        int width = padController.getWidth();
        int center = padX + width / 2;

        if (x > center) {
            padController.moveRight();
        } else {
            padController.moveLeft();
        }
    }
*/
}
