package jp.gr.java_conf.skrb.game.pong;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;

public class PadController {
    private PongFactory factory;

    private Sprite pad;

    private int stageWidth;
    private int stageHeight;

    private int initX;
    private int initY;
    private int step;

    public PadController() {
        factory = PongFactory.getFactory();

	initX = Integer.getInteger("InitPadX").intValue();
	initY = Integer.getInteger("InitPadY").intValue();
	step = Integer.getInteger("PadStep").intValue();
	
	pad = factory.createPad(Integer.getInteger("PadWidth").intValue(),
				Integer.getInteger("PadHeight").intValue());
        pad.moveTo(initX, initY);

	stageWidth = Integer.getInteger("StageWidth").intValue();
        stageHeight = Integer.getInteger("StageHeight").intValue();
    }

    public void init() {
        pad.moveTo(initX, initY);
    }

    public int getX() {
        return pad.getX();
    }

    public int getY() {
        return pad.getY();
    }

    public int getWidth() {
        return pad.getWidth();
    }

    public int getStep() {
        return step;
    }

    public Sprite getPad() {
	return pad;
    }

    public void setPosition(int x, int y) {
        pad.moveTo(x, y);
    }

    public Rectangle getBounds() {
	return new Rectangle(pad.getX(), pad.getY(), pad.getWidth(), pad.getHeight());
    }

    public void moveLeft() {
	int x = pad.getX();
        x -= step;

        if (x < 0) {
            x = 0;
        }

        pad.moveTo(x, pad.getY());
    }

    public void moveRight() {
        int x = pad.getX();
        x += step;

        if (x + pad.getWidth() > stageWidth) {
            x = stageWidth - pad.getWidth();
        }

        pad.moveTo(x, pad.getY());
    }

    public int update(int id, int x, int y) {
        switch(id) {
        case InputEvent.RIGHT:
            moveRight();
            return InputEvent.NOINPUT;
        case InputEvent.LEFT:
            moveLeft();
            return InputEvent.NOINPUT;
        case InputEvent.TAPPED:
            int padX = getX();
            int width = getWidth();

            if (x > padX + width) {
                moveRight();
            } else if (x < padX) {
                moveLeft();
            }

            return InputEvent.TAPPED;
        default:
            return InputEvent.NOINPUT;
        }
    }
}
