package jp.gr.java_conf.skrb.game.pong;

public class PongGameEngine {
    private PongView view;

    private State state;

    private int highScore;

    public PongGameEngine() {
	view = PongFactory.getFactory().getPongView();

        BallController ballController = new BallController();
        PadController padController = new PadController();

	System.out.println("Init");
	state = new DemoState(this, view, padController, ballController);

	view.init();
    }

    public void startGame() {
	do {
	    state = state.execute();
	} while (state != null);

	end();
    }

    private void end() {
	view.end();

	System.out.println("End Game.");
	System.exit(0);
    }

    public int getHighScore() {
	return highScore;
    }

    public void setHighScore(int highScore) {
	this.highScore = highScore;
    }

    public static void usage() {
	System.out.println("java jp.gr.java_conf.skrb.game.pong [option]");
	System.out.println("  Default: Swing version");
	System.out.println("  Option:");
	System.out.println("    -awt         AWT Version");
	System.out.println("    -swing       Swing Version");
	System.out.println("    -fullscreen  Fullscreen AWT Version");
	System.out.println("    -cosmo       PocketCosmo (Personal Java) Version");
	System.out.println("    -help        This message");
    }

    public static void main(String[] args) {
	if (args.length > 0) {
	    if (args[0].equalsIgnoreCase("-awt")){
		jp.gr.java_conf.skrb.game.pong.awt.AWTPongFactory.getFactory();
	    } else if (args[0].equalsIgnoreCase("-swing")){
		jp.gr.java_conf.skrb.game.pong.swing.SwingPongFactory.getFactory();
	    } else if (args[0].equalsIgnoreCase("-fullscreen")){
		jp.gr.java_conf.skrb.game.pong.fullscreen.FSPongFactory.getFactory();
	    } else if (args[0].equalsIgnoreCase("-cosmo")){
		jp.gr.java_conf.skrb.game.pong.cosmo.CPongFactory.getFactory();
	    } else if (args[0].equalsIgnoreCase("-help")){
		usage();
		return;
	    }
	} else {
	    jp.gr.java_conf.skrb.game.pong.swing.SwingPongFactory.getFactory();
	}

	PongGameEngine engine = new PongGameEngine();
	engine.startGame();
    }
}
