package jp.gr.java_conf.skrb.game.pong.cosmo;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Panel;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Vector;

import jp.gr.java_conf.skrb.game.pong.InputEvent;
import jp.gr.java_conf.skrb.game.pong.InputListener;
import jp.gr.java_conf.skrb.game.pong.PongView;
import jp.gr.java_conf.skrb.game.pong.Sprite;

public class CPongView implements PongView {
    private Frame frame;
    private PongPanel panel;
    private Vector listeners;

    private int restBall;
    private int score;
    private int highScore;
    private boolean gameFlag;
    private Sprite ball;
    private Sprite pad;

    private int stageWidth;
    private int stageHeight;
    private int infoY;
    private int scoreX;
    private int highScoreX;
    private int restBallX;
    private int startMessageX;
    private int startMessageY;
    private String startMessage;

    public CPongView() {
	stageWidth = Integer.getInteger("StageWidth").intValue();
	stageHeight = Integer.getInteger("StageHeight").intValue();
	infoY = Integer.getInteger("InfoY").intValue();
	scoreX = Integer.getInteger("ScoreX").intValue();
	highScoreX = Integer.getInteger("HighScoreX").intValue();
	restBallX = Integer.getInteger("RestBallX").intValue();
	startMessageX = Integer.getInteger("StartMessageX").intValue();
	startMessageY = Integer.getInteger("StartMessageY").intValue();
	startMessage = System.getProperty("StartMessage");

        frame = new Frame("Pong");
	frame.setLayout(new BorderLayout());
	frame.setSize(stageWidth, stageHeight);

	panel = new PongPanel();
	frame.add(panel);

	frame.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent event) {
		    fireGameQuitEvent(InputEvent.QUIT);
		}
	    });

	panel.addMouseMotionListener(new MouseMotionAdapter() {
            public void mouseDragged(MouseEvent event) {
		fireGameStartEvent(InputEvent.START);
		firePadMovingEvent(event.getX(), event.getY());
	    }
        });

	listeners = new Vector();
    }

    public void init() {
	frame.setVisible(true);
    }

    public void end() {}

    private synchronized void setRestBall(int rest) {
	this.restBall = rest;
    }

    private synchronized int getRestBall() {
	return restBall;
    }

    private synchronized void setScore(int score) {
	this.score = score;
    }

    private synchronized int getScore() {
	return score;
    }

    private synchronized void setHighScore(int highScore) {
	this.highScore = highScore;
    }

    private synchronized int getHighScore() {
	return highScore;
    }

    private synchronized void setGameFlag(boolean gameFlag) {
	this.gameFlag = gameFlag;
    }

    private synchronized boolean isGaming() {
	return gameFlag;
    }

    private synchronized void setBall(Sprite ball) {
	this.ball = ball;
    }

    private synchronized Sprite getBall() {
	return ball;
    }

    private synchronized void setPad(Sprite pad) {
	this.pad = pad;
    }

    private synchronized Sprite getPad() {
	return pad;
    }

    public void addInputListener(InputListener listener) {
	listeners.addElement(listener);
    }

    public void removeInputListener(InputListener listener) {
	listeners.removeElement(listener);
    }

    private void fireGameStartEvent(int id) {
	InputEvent event = new InputEvent(id);

	for (int i = 0; i < listeners.size(); i++) {
	    InputListener listener = (InputListener)listeners.elementAt(i);
	    listener.gameStart(event);
	}
    }

    private void fireGameQuitEvent(int id) {
	InputEvent event = new InputEvent(id);

	for (int i = 0; i < listeners.size(); i++) {
	    InputListener listener = (InputListener)listeners.elementAt(i);
	    listener.gameQuit(event);
	}
    }

    private void firePadMovingEvent(int id) {
	InputEvent event = new InputEvent(id);

	for (int i = 0; i < listeners.size(); i++) {
	    InputListener listener = (InputListener)listeners.elementAt(i);
	    listener.padMoving(event);
	}
    }

    private void firePadMovingEvent(int x, int y) {
	InputEvent event = new InputEvent(InputEvent.TAPPED, x, y);

	for (int i = 0; i < listeners.size(); i++) {
	    InputListener listener = (InputListener)listeners.elementAt(i);
	    listener.padMoving(event);
	}
    }

    public void draw(int rest, int score, int highScore, Sprite ball, Sprite pad, boolean gameFlag) {
	setRestBall(rest);
	setScore(score);
	setHighScore(highScore);
	setGameFlag(gameFlag);

	setBall(ball);
	setPad(pad);

	panel.repaint();
    }

    class PongPanel extends Panel {
	private Image image;
	private Graphics gg;

	public Dimension getMinimumSize() {
	    return getPreferredSize();
	}

	public Dimension getPreferredSize() {
	    return new Dimension(stageWidth, stageHeight);
	}

	public void update(Graphics g) {
	    paint(g);
	}

	public void paint(Graphics g) {
	    if (image == null) {
		image = createImage(stageWidth, stageHeight);
		gg = image.getGraphics();
	    }

	    gg.setColor(Color.black);
	    gg.fillRect(0, 0, stageWidth, stageHeight);
	    
	    gg.setColor(Color.white);
	    if (!isGaming()) {
		gg.drawString(startMessage, startMessageX, startMessageY);
	    }

	    gg.drawString("Rest: " + getRestBall(), restBallX, infoY);
	    gg.drawString("Score: " + getScore(), scoreX, infoY);
	    gg.drawString("H Score: " + getHighScore(), highScoreX, infoY);
	    
	    Sprite ball = getBall();
	    if (ball != null) {
		ball.draw(gg);
	    }
	    Sprite pad = getPad();
	    if (pad != null) {
		pad.draw(gg);
	    }

	    g.drawImage(image, 0, 0, this);
	}
    }
}
