package jp.gr.java_conf.skrb.game.pong.swing;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferStrategy;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;

import jp.gr.java_conf.skrb.game.pong.InputEvent;
import jp.gr.java_conf.skrb.game.pong.InputListener;
import jp.gr.java_conf.skrb.game.pong.PongView;
import jp.gr.java_conf.skrb.game.pong.Sprite;

public class SwingPongView implements PongView {
    private JFrame frame;
    private PongPanel panel;
    private List listeners;

    private int restBall;
    private int score;
    private int highScore;
    private boolean gameFlag;
    private Sprite ball;
    private Sprite pad;

    private int stageWidth;
    private int stageHeight;
    private int infoY;
    private int scoreX;
    private int highScoreX;
    private int restBallX;
    private int startMessageX;
    private int startMessageY;
    private String startMessage;

    public SwingPongView() {
        stageWidth = Integer.getInteger("StageWidth").intValue();
        stageHeight = Integer.getInteger("StageHeight").intValue();
        infoY = Integer.getInteger("InfoY").intValue();
        scoreX = Integer.getInteger("ScoreX").intValue();
        highScoreX = Integer.getInteger("HighScoreX").intValue();
        restBallX = Integer.getInteger("RestBallX").intValue();
        startMessageX = Integer.getInteger("StartMessageX").intValue();
        startMessageY = Integer.getInteger("StartMessageY").intValue();
        startMessage = System.getProperty("StartMessage");

        frame = new JFrame("Pong");
        frame.setResizable(false);
        frame.setIgnoreRepaint(false);

        panel = new PongPanel();
        panel.setPreferredSize(new Dimension(stageWidth, stageHeight));
        panel.setOpaque(true);
        frame.getContentPane().add(panel);
        ((JComponent)frame.getContentPane()).setOpaque(true);

        frame.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent event) {
                    fireGameQuitEvent(InputEvent.QUIT);
                }
            });

        frame.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent event) {
                int code = event.getKeyCode();
                switch (code) {
                case KeyEvent.VK_Y:
                case KeyEvent.VK_SPACE:
                    fireGameStartEvent(InputEvent.START);
                    break;
                case KeyEvent.VK_Q:
                    fireGameQuitEvent(InputEvent.QUIT);
                    break;
                case KeyEvent.VK_RIGHT:
                    firePadMovingEvent(InputEvent.RIGHT);
                    break;
                case KeyEvent.VK_LEFT:
                    firePadMovingEvent(InputEvent.LEFT);
                    break;
                } 
            }
        });

        listeners = Collections.synchronizedList(new ArrayList());
    }

    public void init() {
        frame.pack();
        frame.setVisible(true);
    }

    private synchronized void setRestBall(int rest) {
        this.restBall = rest;
    }

    private synchronized int getRestBall() {
        return restBall;
    }

    private synchronized void setScore(int score) {
        this.score = score;
    }

    private synchronized int getScore() {
        return score;
    }

    private synchronized void setHighScore(int highScore) {
        this.highScore = highScore;
    }

    private synchronized int getHighScore() {
        return highScore;
    }

    private synchronized void setGameFlag(boolean gameFlag) {
        this.gameFlag = gameFlag;
    }

    private synchronized boolean isGaming() {
        return gameFlag;
    }

    private synchronized void setBall(Sprite ball) {
        this.ball = ball;
    }

    private synchronized Sprite getBall() {
        return ball;
    }

    private synchronized void setPad(Sprite pad) {
        this.pad = pad;
    }

    private synchronized Sprite getPad() {
        return pad;
    }

    public void addInputListener(InputListener listener) {
        listeners.add(listener);
    }

    public void removeInputListener(InputListener listener) {
        listeners.remove(listener);
    }

    private void fireGameStartEvent(int id) {
        InputEvent event = new InputEvent(id);

        for (int i = 0; i < listeners.size(); i++) {
            InputListener listener = (InputListener)listeners.get(i);
            listener.gameStart(event);
        }
    }

    private void fireGameQuitEvent(int id) {
        InputEvent event = new InputEvent(id);

        for (int i = 0; i < listeners.size(); i++) {
            InputListener listener = (InputListener)listeners.get(i);
            listener.gameQuit(event);
        }
    }

    private void firePadMovingEvent(int id) {
        InputEvent event = new InputEvent(id);

        for (int i = 0; i < listeners.size(); i++) {
            InputListener listener = (InputListener)listeners.get(i);
            listener.padMoving(event);
        }
    }

    public void draw(int rest, int score, int highScore, Sprite ball, Sprite pad, boolean gameFlag) {
        setRestBall(rest);
        setScore(score);
        setHighScore(highScore);
        setGameFlag(gameFlag);

        setBall(ball);
        setPad(pad);

        panel.repaint();
    }

    public void end() {}

    class PongPanel extends JPanel {
        private void drawInfo(Graphics g) {
            g.setColor(Color.WHITE);
            if (!isGaming()) {
                g.drawString(startMessage, startMessageX, startMessageY);
            }
            
            g.drawString("Rest Ball: " + getRestBall(), restBallX, infoY);
            g.drawString("Score: " + getScore(), scoreX, infoY);
            g.drawString("High Score: " + getHighScore(), highScoreX, infoY);
        }

        public void paintComponent(Graphics g) {
            g.setColor(Color.BLACK);
            g.fillRect(0, 0, stageWidth, stageHeight);
        
            drawInfo(g);

            Sprite ball = getBall();
            if (ball != null) {
                ball.draw(g);
            }
            Sprite pad = getPad();
            if (pad != null) {
                pad.draw(g);
            }
        }
    }
}
